# vim: set fileencoding=utf-8 :
"""
PermissionHandler registry


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
License:
    The MIT License (MIT)

    Copyright (c) 2012 Alisue allright reserved.

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
    IN THE SOFTWARE.

"""
from __future__ import with_statement
from django.conf import settings
from django.db.models.base import ModelBase
from django.core.exceptions import ImproperlyConfigured

from permission.exceptions import AlreadyRegistered
from permission.exceptions import NotRegistered
from permission.handlers.base import PermissionHandler

__all__ = ('registry', 'PermissionHandler',)


class Registry(object):
    """Registry of permission handlers

    Register permission handlers to this registry to use.

    """
    def __init__(self):
        self._registry = {}

    def register(self, model_or_iterable, handler_class):
        """register new permission handler for the model(s)

        If the model is already registered in registry,
        ``AlreadyRegistered`` exception will come up.

        """
        # Don't import the humonogus validation code unless required
        if settings.DEBUG:
            from permission.validation import validate
        else:
            validate = lambda model, mediator: None

        if not hasattr(model_or_iterable, '__iter__'):
            # this mean subclass of models.Model
            model_or_iterable = [model_or_iterable]
        for model in model_or_iterable:
            # Validate (which might be a no-op)
            validate(handler_class, model)

            if model._meta.abstract:
                raise ImproperlyConfigured(
                        'The model %s is abstract, so it cannot be registered '
                        'with permission.' % model.__name__)
            if model in self._registry:
                raise AlreadyRegistered(model)

            # Instantiate the handler to save in the registry
            instance = handler_class(model)
            self._registry[model] = instance

    def unregister(self, model_or_iterable):
        if isinstance(model_or_iterable, ModelBase):
            model_or_iterable = [model_or_iterable]
        for model in model_or_iterable:
            if model not in self._registry:
                raise NotRegistered(model)
            # remove from registry
            del self._registry[model]

    def get_handlers(self):
        return tuple(self._registry.values())

    def get_module_handlers(self, app_label):
        return tuple()

registry = Registry()



# vim: set fileencoding=utf-8 :
"""
templatetags


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
License:
    The MIT License (MIT)

    Copyright (c) 2012 Alisue allright reserved.

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
    IN THE SOFTWARE.

"""
from django.conf import settings
from django import template
from django.template import TemplateSyntaxError
from django.template import VariableDoesNotExist
from django.template.smartif import infix
from django.template.smartif import IfParser
from django.template.smartif import OPERATORS
from django.template.defaulttags import Node
from django.template.defaulttags import NodeList
from django.template.defaulttags import TemplateLiteral

from permission.templatetags.patch import IfNode
from permission.templatetags.patch import parser_patch

register = template.Library()

def of_operator(context, x, y):
    return x.eval(context), y.eval(context)

def has_operator(context, x, y):
    user = x.eval(context)
    perm = y.eval(context)
    if isinstance(perm, (list, tuple)):
        perm, obj = perm
    else:
        obj = None
    return user.has_perm(perm, obj)

EXTRA_OPERATORS = {
    'of': infix(20, of_operator),
    'has': infix(10, has_operator),
}
EXTRA_OPERATORS.update(OPERATORS)
# Assign 'id' to each:
for key, op in EXTRA_OPERATORS.items():
    op.id = key

class PermissionIfParser(IfParser):
    OPERATORS = EXTRA_OPERATORS

    def translate_token(self, token):
        try:
            op = self.OPERATORS[token]
        except (KeyError, TypeError):
            return self.create_var(token)
        else:
            return op()

class TemplatePermissionIfParser(PermissionIfParser):
    error_class = TemplateSyntaxError

    def __init__(self, parser, *args, **kwargs):
        self.template_parser = parser
        super(TemplatePermissionIfParser, self).__init__(*args, **kwargs)

    def create_var(self, value):
        return TemplateLiteral(self.template_parser.compile_filter(value), value)


@register.tag('permission')
def do_permissionif(parser, token):
    # patch parser for django 1.3.1
    parser = parser_patch(parser)

    bits = token.split_contents()
    ELIF = "el%s" % bits[0]
    ELSE = "else"
    ENDIF = "end%s" % bits[0]

    # {% if ... %}
    bits = bits[1:]
    condition = do_permissionif.Parser(parser, bits).parse()
    nodelist = parser.parse((ELIF, ELSE, ENDIF))
    conditions_nodelists = [(condition, nodelist)]
    token = parser.next_token()

    # {% elif ... %} (repeatable)
    while token.contents.startswith(ELIF):
        bits = token.split_contents()[1:]
        condition = do_permissionif.Parser(parser, bits).parse()
        nodelist = parser.parse((ELIF, ELSE, ENDIF))
        conditions_nodelists.append((condition, nodelist))
        token = parser.next_token()

    # {% else %} (optional)
    if token.contents == ELSE:
        nodelist = parser.parse((ENDIF,))
        conditions_nodelists.append((None, nodelist))
        token = parser.next_token()

    # {% endif %}
    assert token.contents == ENDIF

    return IfNode(conditions_nodelists)
do_permissionif.Parser = TemplatePermissionIfParser

if settings.PERMISSION_REPLACE_BUILTIN_IF:
    register.tag('if', do_permissionif)




# vim: set fileencoding=utf-8 :
"""
Permission handler base


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
License:
    The MIT License (MIT)

    Copyright (c) 2012 Alisue allright reserved.

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
    IN THE SOFTWARE.

"""
from __future__ import with_statement
import django
from django.db import models

from django.contrib.auth.models import Permission
from django.utils.translation import ugettext_lazy as _

from mptt.models import MPTTModel
from mptt.models import TreeForeignKey
from mptt.models import TreeManager

if django.VERSION[1] > 4:
    User = django.contrib.auth.get_user_model()
else:
    from django.contrib.auth.models import User

class RoleManager(TreeManager):
    def get_by_natural_key(self, codename):
        return self.get(codename=codename)

    def filter_by_user(self, user_obj):
        """return queryset of roles which ``user_obj`` have"""
        # do not defer anything otherwise the returning queryset
        # contains defered instance.
        roles_qs = self.none()
        for role in self.filter(_users=user_obj).iterator():
            roles_qs |= role.get_ancestors()
            roles_qs |= self.filter(pk=role.pk)
        return roles_qs

    def get_all_permissions_of_user(self, user_obj):
        """get a set of all permissions of ``user_obj``"""
        # name, parent, _users, _permissions are required.
        roles = self.defer('codename', 'description').filter(_users=user_obj)
        permissions = Permission.objects.none()
        for role in roles.iterator():
            permissions |= role.permissions
        return permissions.distinct()


class Role(MPTTModel):
    """A role model for enhanced permission system."""
    name = models.CharField(_('name'), max_length=50)
    codename = models.CharField(_('codename'), max_length=100, unique=True)
    description = models.TextField(_('description'), blank=True,
            help_text=_('A description of this permission role'))

    parent = TreeForeignKey('self', verbose_name=_('parent role'),
            related_name='children', blank=True, null=True)

    _users = models.ManyToManyField(User, verbose_name=_('user'),
            related_name='_roles', db_column='users', blank=True)

    _permissions = models.ManyToManyField(Permission, verbose_name=_('permissions'),
            related_name='roles', db_column='permissions', blank=True)

    objects = RoleManager()

    class Meta:
        verbose_name = _('role')
        verbose_name_plural = _('roles')

    class MPTTMeta:
        order_insertion_by = ['name']

    def __unicode__(self):
        return self.name

    def natural_key(self):
        return (self.codename,)

    @property
    def users(self):
        """get all users who belongs to this role or superroles"""
        role_pks = self.get_descendants(True).values_list('id', flat=True)
        qs = User.objects.only('id', '_roles').filter(_roles__pk__in=role_pks).distinct()
        qs = qs.defer(None)
        def remove_role_perm_cache_before(fn):
            def inner(manager, *objs):
                for obj in objs:
                    if hasattr(obj, '_role_perm_cache'):
                        delattr(obj, '_role_perm_cache')
                return fn(manager, *objs)
            return inner
        # add methods
        qs.add = remove_role_perm_cache_before(self._users.add)
        qs.remove = remove_role_perm_cache_before(self._users.remove)
        qs.clear = self._users.clear
        return qs

    @property
    def permissions(self):
        """get all permissions which this role or subroles have"""
        role_pks = list(self.get_ancestors().values_list('id', flat=True))
        role_pks.append(self.pk)
        qs = Permission.objects.only('id', 'roles').filter(roles__pk__in=role_pks).distinct()
        qs = qs.defer(None)
        # add methods
        qs.add = self._permissions.add
        qs.remove = self._permissions.remove
        qs.clear = self._permissions.clear
        return qs

    def is_belong(self, user_obj):
        """whether the ``user_obj`` belongs to this role or superroles"""
        return self.users.filter(pk=user_obj.pk).exists()


def get_permission_instance(str_or_instance):
    """get permission instance from string or instance"""
    if isinstance(str_or_instance, basestring):
        app_label, codename = str_or_instance.split('.', 1)
        instance = Permission.objects.get(
                content_type__app_label=app_label,
                codename=codename
            )
    else:
        instance = str_or_instance
    return instance



import inspect
from django.db.models import Model

from class_decorators import permission_required as class_permission_required
from method_decorators import permission_required as method_permission_required

__all__ = ['permission_required']


def permission_required(perm, queryset_or_model=None, login_url=None, raise_exception=False):
    """Permission check decorator for classbased/functional generic view

    This decorator works as class, method or function decorator without modification.
    DO NOT use ``method_decorator`` or whatever whilte this decorator will use
    ``self`` argument for method of classbased generic view.

    Arguments:

        perm
            permission string

        queryset_or_model
            a queryset or model for finding object. With classbased generic
            view, ``None`` for using view default queryset. When the view does
            not define ``get_queryset``, ``queryset``, ``get_object`` and ``object``
            then ``obj=None`` is used to check permission. With functional generic
            view, ``None`` for using passed queryset. When non queryset was passed
            then ``obj=None`` is used to check permission.

    Usage:

        # As class decorator
        @permission_required('auth.change_user')
        class UpdateAuthUserView(UpdateView):
            # ...

        # As method decorator
        class UpdateAuthUserView(UpdateView):
            @permission_required('auth.change_user')
            def dispatch(self, request, *args, **kwargs):
                # ...
        
        # As function decorator
        @permission_required('auth.change_user')
        def update_auth_user(request, *args, **kwargs):
            # ...

    .. Note::
        Classbased generic view is recommended while you can regulate the queryset
        with ``get_queryset()`` method. Detecting object from passed kwargs may
        not work correctly.

    """
    # convert model to queryset
    if queryset_or_model and issubclass(queryset_or_model, Model):
        queryset_or_model = queryset_or_model._default_manager.all()

    def wrapper(class_or_method):
        if inspect.isclass(class_or_method):
            decorator = class_permission_required
        else:
            # method_permission_required can handle method or function
            # correctly.
            decorator = method_permission_required
        return decorator(perm, queryset_or_model, login_url, raise_exception)(class_or_method)
    return wrapper



#!/usr/bin/env python
# vim: set fileencoding=utf-8 :
"""
Utils used in decorators


AUTHOR:
    lambdalisue[Ali su ae] (lambdalisue@hashnote.net)
    
License:
    The MIT License (MIT)

    Copyright (c) 2012 Alisue allright reserved.

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to
    deal in the Software without restriction, including without limitation the
    rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
    sell copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
    FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
    IN THE SOFTWARE.

"""
from __future__ import with_statement
import time
import datetime
import urlparse
from django.conf import settings
from django.http import Http404
from django.db.models.fields import DateTimeField
from django.shortcuts import get_object_or_404
from django.contrib.auth import REDIRECT_FIELD_NAME
from django.views.generic.edit import BaseCreateView
from permission.exceptions import ValidationError

try:
    from django.utils import timezone
    datetime_now = timezone.now
except ImportError:
    datetime_now = datetime.datetime.now

def redirect_to_login(request, login_url=None, redirect_field_name=REDIRECT_FIELD_NAME):
    """redirect to login"""
    path = request.build_absolute_uri()
    # if the login url is the same scheme and net location then just
    # use the path as the "next" url.
    login_scheme, login_netloc = urlparse.urlparse(login_url or settings.LOGIN_URL)[:2]
    current_scheme, current_netloc = urlparse.urlparse(path)[:2]
    if ((not login_scheme or login_scheme == current_scheme) and
        (not login_netloc or login_netloc == current_netloc)):
        path = request.get_full_path()
    from django.contrib.auth.views import redirect_to_login as auth_redirect_to_login
    return auth_redirect_to_login(path, login_url, redirect_field_name)

def get_object_from_classbased_instance(instance, queryset, request, *args, **kwargs):
    """get object from an instance of classbased generic view"""
    # initialize request, args, kwargs of classbased_instance
    # most of methods of classbased view assumed these attributes
    # but these attributes is initialized in ``dispatch`` method.
    instance.request = request
    instance.args = args
    instance.kwargs = kwargs

    # get queryset from class if ``queryset_or_model`` is not specified
    if hasattr(instance, 'get_queryset') and not queryset:
        queryset = instance.get_queryset()
    elif hasattr(instance, 'queryset') and not queryset:
        queryset = instance.queryset
    elif hasattr(instance, 'model') and not queryset:
        queryset = instance.model._default_manager.all()
        
    # get object
    if hasattr(instance, 'get_object'):
        try:
            obj = instance.get_object(queryset)
        except AttributeError, e:
            # CreateView has ``get_object`` method but CreateView
            # should not have any object before thus simply set
            # None
            if isinstance(instance, BaseCreateView):
                obj = None
            else:
                raise e
    elif hasattr(instance, 'object'):
        obj = instance.object
    else:
        obj = None
    return obj

def get_object_from_list_detail_view(request, *args, **kwargs):
    """get object from generic list_detail.detail view"""
    queryset = kwargs['queryset']
    object_id = kwargs.get('object_id', None)
    slug = kwargs.get('slug', None)
    slug_field = kwargs.get('slug_field', 'slug')
    if object_id:
        obj = get_object_or_404(queryset, pk=object_id)
    elif slug and slug_field:
        obj = get_object_or_404(queryset, **{slug_field: slug})
    else:
        raise AttributeError(
                "Generic detail view must be called with either an "
                "object_id or a slug/slug_field."
            )
    return obj
def _get_object_from_list_detail_view_validation(request, *args, **kwargs):
    if 'queryset' not in kwargs:
        return False
    elif 'object_id' not in kwargs and 'slug' not in kwargs:
        return False
    return True
get_object_from_list_detail_view.validate = _get_object_from_list_detail_view_validation


def get_object_from_date_based_view(request, *args, **kwargs):
    """get object from generic date_based.detail view"""
    year, month, day = kwargs['year'], kwargs['month'], kwargs['day']
    month_format = kwargs.get('month_format', '%b')
    day_format = kwargs.get('day_format', '%d')
    date_field = kwargs['date_field']
    queryset = kwargs['queryset']
    object_id = kwargs.get('object_id', None)
    slug = kwargs.get('slug', None)
    slug_field = kwargs.get('slug_field', 'slug')
    
    try:
        tt = time.strptime(
                '%s-%s-%s' % (year, month, day),
                '%s-%s-%s' % ('%Y', month_format, day_format)
            )
        date = datetime.date(*tt[:3])
    except ValueError:
        raise Http404

    model = queryset.model

    if isinstance(model._meta.get_field(date_field), DateTimeField):
        lookup_kwargs = {
                '%s__range' % date_field: (
                    datetime.datetime.combine(date, datetime.time.min),
                    datetime.datetime.combine(date, datetime.time.max),
                )}
    else:
        lookup_kwargs = {date_field: date}

    now = datetime_now()
    if date >= now.date() and not kwargs.get('allow_future', False):
        lookup_kwargs['%s__lte' % date_field] = now
    if object_id:
        lookup_kwargs['pk'] = object_id
    elif slug and slug_field:
        lookup_kwargs['%s__exact' % slug_field] = slug
    else:
        raise AttributeError(
                "Generic detail view must be called with either an "
                "object_id or a slug/slug_field."
            )
    return get_object_or_404(queryset, **lookup_kwargs)
def _get_object_from_date_based_view_validation(request, *args, **kwargs):
    if 'queryset' not in kwargs:
        return False
    elif 'year' not in kwargs or 'month' not in kwargs or 'day' not in kwargs:
        return False
    elif 'object_id' not in kwargs and 'slug' not in kwargs:
        return False
    return True
get_object_from_date_based_view.validate = _get_object_from_date_based_view_validation
